@@ -2,8 +2,8 @@
 from __future__ import absolute_import, unicode_literals, division
 import six
 import logging
-from collections import Sequence
-from armet import http
+from collections import Sequence, MutableSequence, Iterable
+from armet import http, pagination
 from armet.exceptions import ValidationError
 from armet.resources.resource import base
 
@@ -89,7 +89,7 @@ class ManagedResource(base.Resource):
         if not set(args).issubset(self.allowed_operations):
             raise http.exceptions.Forbidden()
 
-    def make_response(self, data=None, status=http.client.OK):
+    def make_response(self, data=None):
         """Fills the response object from the passed data."""
         if data is not None:
             # Prepare the data for transmission.
@@ -98,9 +98,6 @@ class ManagedResource(base.Resource):
             # Encode the data using a desired encoder.
             self.response.write(data, serialize=True)
 
-        # Make sure that the status code is set.
-        self.response.status = status
-
     def prepare(self, data):
         if data is None:
             # No data; return nothing.
@@ -111,8 +108,12 @@ class ManagedResource(base.Resource):
             if '/' in self.path:
                 raise http.exceptions.NotFound()
 
-        if (isinstance(data, Sequence)
+        if (isinstance(data, Iterable)
                 and not isinstance(data, six.string_types)):
+            # Resolve the sequence only if we need to.
+            if not isinstance(data, MutableSequence):
+                data = list(data)
+
             # Attempt to prepare each item of the iterable (as long as
             # we're not a string or some sort of mapping).
             for index, value in enumerate(data):
@@ -284,6 +285,11 @@ class ManagedResource(base.Resource):
             if not items:
                 raise http.exceptions.NotFound()
 
+        if (isinstance(items, Iterable)
+                and not isinstance(items, six.string_types)) and items:
+            # Paginate over the collection.
+            items = pagination.paginate(self.request, self.response, items)
+
         # Build the response object.
         self.make_response(items)
 
@@ -303,7 +309,8 @@ class ManagedResource(base.Resource):
         item = self.create(data)
 
         # Build the response object.
-        self.make_response(item, status=http.client.CREATED)
+        self.response.status = http.client.CREATED
+        self.make_response(item)
 
     def put(self, request, response):
         """Processes a `PUT` request."""
@@ -330,7 +337,7 @@ class ManagedResource(base.Resource):
                 raise http.exceptions.NotImplemented()
 
             # Build the response object.
-            self.make_response(target, status=http.client.OK)
+            self.make_response(target)
 
         else:
             # Ensure we're allowed to create the resource.
@@ -340,7 +347,8 @@ class ManagedResource(base.Resource):
             target = self.create(data)
 
             # Build the response object.
-            self.make_response(target, status=http.client.CREATED)
+            self.response.status = http.client.CREATED
+            self.make_response(target)
 
     def delete(self, request, response):
         """Processes a `DELETE` request."""
@@ -355,4 +363,5 @@ class ManagedResource(base.Resource):
         self.destroy()
 
         # Build the response object.
-        self.make_response(status=http.client.NO_CONTENT)
+        self.response.status = http.client.NO_CONTENT
+        self.make_response()
